/* AMX Mod X
*   Grenade Jump (fun modification for CS)
*
* (c) Copyright 2006 by VEN
*
* This file is provided as is (no warranties)
*
*     DESCRIPTION
*       Plugin provides an additional velocity to a player on grenade explosion.
*
*     INFORMATION
*       I know that there are another nade jump plugins but i made this one by myself and
*       didn't looked into other plugins' code, so this one came up as quite original one.
*       I noticed that some of such plugins does not ballance the gameplay ascpect at all.
*       This one on the contrary have original features that allows to ballance gameplay.
*
*     FEATURES
*       - configurable velocity multiplier - useful since it is affect jump physics
*       - falldamage mode (on/off) - useful to keep health on high jumps
*       - recovery modes - allows safe jumps but also keeps default nade gameplay purpose
*       - health/grenade recovery - useful to keep health/grenade for the further actions
*
*     CVARS
*       gj_velocity (N[.F]: value, default: 10.0, 0: disable the plugin) - velocity multiplier
*       gj_falldmg (0: OFF, 1: ON, default: 0) - falldamage mode (for grenade jumps only)
*       gj_recover (0: OFF, N: ON, default: 1) - controls how health/grenade recovery is act
*         1 - if damage done only to the grenade owner and not other players
*         2 - if damage done only to the grenade owner and/or its teammates and not enemies
*         3 - if damage done to any player(s)
*         Note: if at least one player is killed by the grenade, hp/nade will not be recovered
*       gj_reverthp (0: OFF, 1: ON, default: 1) - reverts players health on grenade damage
*       gj_givenade (0: OFF, 1: ON, default: 1) - gives back hegrenade to the grenade owner
*
*     EXPLANATION
*       The information above isn't enough so here is the explanation for the default CVARs.
*       Falldamage will be disabled only on "true" nade jumps, i.e. if damage done in jump.
*       It's possible to keep self-jumping but also you can throw the nade and kill someone.
*       If you want to keep self-jumping you have to make sure that no players are around.
*       In this case your health and grenade will be restored back so you can repeat a jump.
*       But also you should note that since the actual damage isn't blocked but only health
*       is restored you better have medium amount of health + vest or high amount of health.
*       If you plan to do a damage or kill someone, throw the nade closer to the victim but
*       remember that hp/nade recovery is disabled in this case so make sure you aren't close.
*
*     CREDITS
*       Team17 - for the game "Worms", where i first did saw the nade jump (very long time ago)
*       leakgfhp - postthink watertype falldamage elimination method
*/

#include <amxmodx>
#include <fakemeta>
#include <xs>

// plugin's main information
#define PLUGIN_NAME "Grenade Jump"
#define PLUGIN_VERSION "0.1"
#define PLUGIN_AUTHOR "VEN"

enum {
	recover_none,
	recover_owner,
	recover_team,
	recover_all
}

new g_pcvar_velocity
new g_pcvar_falldmg
new g_pcvar_recover
new g_pcvar_reverthp
new g_pcvar_givenade

new g_ipsz_hegrenade

new g_maxplayers

#define MAX_PLAYERS 32
new bool:g_no_falldmg[MAX_PLAYERS + 1]
new bool:g_no_recover[MAX_PLAYERS + 1]
new g_damage[MAX_PLAYERS + 1][MAX_PLAYERS + 1]
new g_empty[MAX_PLAYERS + 1]

public plugin_init() {
	register_plugin(PLUGIN_NAME, PLUGIN_VERSION, PLUGIN_AUTHOR)

	register_event("Damage", "event_damage", "b", "2!0")

	register_forward(FM_PlayerPostThink, "forward_player_postthink")

	g_pcvar_velocity = register_cvar("gj_velocity", "10.0")
	g_pcvar_falldmg = register_cvar("gj_falldmg", "0")
	g_pcvar_recover = register_cvar("gj_recover", "1")
	g_pcvar_reverthp = register_cvar("gj_reverthp", "1")
	g_pcvar_givenade = register_cvar("gj_givenade", "1")

	g_ipsz_hegrenade = engfunc(EngFunc_AllocString, "weapon_hegrenade")

	g_maxplayers = get_maxplayers()
}

public event_damage(id) {
	new Float:mul = get_pcvar_float(g_pcvar_velocity)
	if (mul <= 0)
		return

	new wid, id2 = get_user_attacker(id, wid)
	if (wid != CSW_HEGRENADE)
		return

	new dmg = read_data(2)
	new bool:alive = bool:is_user_alive(id)
	if (alive) {
		new ent = pev(id, pev_dmg_inflictor)
		static Float:v1[3], Float:v2[3], Float:v3[3]
		pev(id, pev_origin, v1)
		pev(ent, pev_origin, v2)
		xs_vec_sub(v1, v2, v3)
		xs_vec_normalize(v3, v3)
		xs_vec_mul_scalar(v3, mul * dmg, v3)

		pev(id, pev_velocity, v1)
		pev(ent, pev_velocity, v2)
		xs_vec_add(v1, v2, v1)
		xs_vec_add(v1, v3, v1)
		set_pev(id, pev_velocity, v1)

		if (!get_pcvar_num(g_pcvar_falldmg) && !(pev(id, pev_flags) & FL_ONGROUND))
			g_no_falldmg[id] = true
	}

	new recover = get_pcvar_num(g_pcvar_recover)
	if (recover <= recover_none)
		return

	new reverthp = get_pcvar_num(g_pcvar_reverthp)
	new givenade = get_pcvar_num(g_pcvar_givenade)
	if (reverthp)
		g_damage[id2][id] = dmg
	else if (!givenade)
		return

	if (!task_exists(id2)) {
		static param[2]
		param[0] = reverthp
		param[1] = givenade
		set_task(0.1, "task_recover", id2, param, 2)
	}

	if (!alive || !(id == id2 || recover == recover_all || (recover == recover_team && get_user_team(id) == get_user_team(id2))))
		g_no_recover[id2] = true
}

public task_recover(param[2], id) {
	if (g_no_recover[id]) {
		g_no_recover[id] = false
		g_damage[id] = g_empty
		return
	}

	if (param[0]) {
		new Float:health, dmg
		for (new i = 1; i <= g_maxplayers; ++i) {
			if (!is_user_alive(i))
				continue

			dmg = g_damage[id][i]
			if (!dmg)
				continue

			pev(i, pev_health, health)
			set_pev(i, pev_health, health + dmg)
		}

		g_damage[id] = g_empty
	}

	if (!param[1] || !is_user_alive(id))
		return

	new ent = engfunc(EngFunc_CreateNamedEntity, g_ipsz_hegrenade)
	if (!pev_valid(ent))
		return

	static Float:origin[3]
	pev(id, pev_origin, origin)
	set_pev(ent, pev_origin, origin)
	set_pev(ent, pev_spawnflags, pev(ent, pev_spawnflags) | SF_NORESPAWN)
	dllfunc(DLLFunc_Spawn, ent)

	new save = pev(ent, pev_solid)
	dllfunc(DLLFunc_Touch, ent, id)
	if (pev(ent, pev_solid) != save)
		return

	engfunc(EngFunc_RemoveEntity, ent)

	return
}

public forward_player_postthink(id) {
	if (g_no_falldmg[id] && pev(id, pev_flags) & FL_ONGROUND) {
		set_pev(id, pev_watertype, CONTENTS_WATER)
		g_no_falldmg[id] = false
	}
}

public client_disconnect(id) {
	g_no_falldmg[id] = false
}
